\name{LDSM}
\alias{LDSM}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
LD Sparse Matrix
}
\description{
Compute and save the LD sparse matrix (R.mat) using reference genotype. Sparse "Matrix" package is used to construct matrix.
}
\usage{
LDSM(geno.x, geno.map, ld.mode, win.size, refld.bed = "gcm_band")
}
%- maybe also 'usage' for other objects documented here.
\arguments{
\item{geno.x}{N * M reference genotype matrix.}
\item{geno.map}{Reference genotype map, here plink.bim can be directly used.}
\item{ld.mode}{There are three option:gcm_band, Mb_band and ref_block. The default is "gcm_band" which will produce a banded matrix base on genetic distance (cm). Mb_band will produce a banded matrix base on postion (Mb). ref_block will produce a block matrix based on reference LD blocks.}
\item{win.size}{Bandwidth for the banded matrix.}
\item{refld.bed}{Reference LD blocks in human populations, which contain columns: CHR Start Stop.}
}
\details{
The estimation procedure requires individual-level genotype data for the computation of the LD matrix Ω and the correlation matrix R. When individual-level genotypes are not available, a suitable reference panel can be used for the computation of Ω and R. Genotypes from the 1000 genomes project can be used as the genotype reference panel. To further reduce computational cost and memory requirement, a banded (or block) matrix plus a low rank matrix to approximate Ω and R for each chromosome separately. In particular, we computed Ω ̂ and R ̂ in the reference panel, extracted the banded parts (Ω_s  and R_s) using a bandwidth of 1cM, and added a one-rank matrix (Ω_LR) with equal element 1/n to Ω_s   to ensure that the off-diagonal elements in the approximated Ω matrix equal its expectation.
}
\value{
\item{Rmat}{R matrix.}
\item{Omat}{Omega matrix.}
\item{ldsc}{ld score for each SNP.}
}
\references{

}
\author{

}
\note{
%%  ~~further notes~~
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
}
\examples{
## Download some example data from http://www.xzlab.org/software.html
##

setwd("C:/Users/xingjie hao/Desktop/SMART_example")
libarary(Matrix)
library(data.table)

## load reference genotype, we can direcly use the PLINK format
chr_index <- 1
gmat <- fread(paste("C:/Users/xingjie hao/Desktop/SMART_example/Ref1K/gtype/ref1k_chr_", chr_index, ".raw", sep = ""), header = T)
gmap <- fread(paste("C:/Users/xingjie hao/Desktop/SMART_example/Ref1K/gtype/ref1k_chr_", chr_index, ".bim", sep = ""), header = F)
gmat <- as.data.frame(gmat)
gmat <- gmat[, -c(1:6)]
gmat <- as.matrix(gmat)
gmat <- scale(gmat)
gmat <- unname(gmat)
gmap <- as.data.frame(gmap)[, 1:4]
colnames(gmap) <- c("CHR", "SNP", "cM", "BP")

## Compute the LD Sparse Matrix
ldmat_band <- LDSM(geno.x = gmat, geno.map = gmap, ld.mode = "gcm_band", win.size = 1, refld.bed = NULL)
R.mat <- ldmat_band$Rmat
## Save the LD Sparse Matrix
save(R.mat, file = paste("C:/Users/xingjie hao/Desktop/SMART_example/Ref1K/LDmat/ldsmat_", chr_index, ".RData", sep = ""))
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ~kwd1 }% use one of  RShowDoc("KEYWORDS")
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line
